@@ -102,6 +102,7 @@ typedef struct appdata_s {
 
   struct {
     char *import_path;
+    GtkWidget *menu_track;
     GtkWidget *menu_item_import;
     GtkWidget *menu_item_export;
     GtkWidget *menu_item_clear;
@@ -111,9 +112,15 @@ typedef struct appdata_s {
     canvas_item_t *gps_item;      // the purple curcle
   } track;
 
+  GtkWidget *menu_item_project_close;
+
+  GtkWidget *menu_osm;
+
+  GtkWidget *menu_wms;
   GtkWidget *menu_item_wms_clear;
   GtkWidget *menu_item_wms_adjust;
 
+  GtkWidget *menu_map;
   GtkWidget *menu_item_map_hide_sel;
   GtkWidget *menu_item_map_show_all;
 
