@@ -366,7 +366,7 @@ static GtkWidget *tag_widget(tag_context_t *context) {
   		     GTK_SIGNAL_FUNC(on_tag_last), context);
 
   GtkWidget *presets = josm_presets_select(context->appdata, context);
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), presets);
+  if(presets) gtk_box_pack_start_defaults(GTK_BOX(hbox), presets);
 
   context->but_add = gtk_button_new_with_label(_("Add..."));
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_add);
@@ -488,13 +488,17 @@ void info_dialog(GtkWidget *parent, appdata_t *appdata, relation_t *relation) {
       label = gtk_label_new(nodes_str);
       gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
       g_free(nodes_str);
-      
-      if(appdata->map->selected.way->draw_flags & OSM_DRAW_FLAG_POLYGON) 
-	label = gtk_label_new("Closed way (polygon)");
-      else
-	label = gtk_label_new("Open way (line)");
-      
+
+      char *type_str = g_strdup_printf("%s (%s)",
+	 (osm_way_get_last_node(appdata->map->selected.way) == 
+	  osm_way_get_first_node(appdata->map->selected.way))?
+				       "closed way":"open way",
+	 (appdata->map->selected.way->draw.flags & OSM_DRAW_FLAG_AREA)?
+				       "area":"line");
+ 
+      label = gtk_label_new(type_str);      
       gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+      g_free(type_str);
     }
   } else {
     /* relations tell something about their members */
