@@ -22,8 +22,74 @@
 
 #include "appdata.h"
 
+/* disable/enable main screen control dependant on presence of open project */
+static void main_project_loaded(appdata_t *appdata) {
+  gboolean loaded = (appdata->project != NULL);
+
+  /* cancel any action in progress */
+  if(GTK_WIDGET_FLAGS(appdata->iconbar->cancel) & GTK_SENSITIVE) 
+    map_action_cancel(appdata);
+
+  /* ---- set project name as window title ----- */
+#ifndef USE_HILDON
+  char *str = NULL;
+  if(loaded) 
+    str = g_strdup_printf("OSM2Go - %s", appdata->project->name);
+  else 
+    str = g_strdup_printf("OSM2Go");
+    
+  gtk_window_set_title(GTK_WINDOW(appdata->window), str);
+  g_free(str);
+#else
+  if(loaded) 
+    gtk_window_set_title(GTK_WINDOW(appdata->window), appdata->project->name);
+  else
+    gtk_window_set_title(GTK_WINDOW(appdata->window), "");
+#endif
+
+  if(appdata->iconbar && appdata->iconbar->toolbar)
+    gtk_widget_set_sensitive(appdata->iconbar->toolbar, loaded);
+
+  /* disable all menu entries related to map */
+  gtk_widget_set_sensitive(appdata->track.menu_track, loaded);
+  gtk_widget_set_sensitive(appdata->menu_osm, loaded);
+  gtk_widget_set_sensitive(appdata->menu_wms, loaded);
+  gtk_widget_set_sensitive(appdata->menu_map, loaded);
+  gtk_widget_set_sensitive(appdata->menu_item_project_close, loaded);
+
+  if(!loaded)
+    statusbar_set(appdata, _("Plase load or create a project"), FALSE);
+}
+
 /******************** begin of menu *********************/
 
+#if 0 // simplify menu
+static struct {
+  enum { MENU_END, MENU_ITEM, MENU_SUB, MENU_SUB_END, MENU_SEP }  type;
+
+  char *title;
+  GCallback c_handler;
+} menu[] = {
+  { MENU_SUB, "OSM", NULL },
+
+  { MENU_END,  NULL, NULL },
+};
+#endif
+
+static void 
+cb_menu_project_open(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  project_load(appdata, NULL);
+  main_project_loaded(appdata);
+}
+
+static void 
+cb_menu_project_close(GtkWidget *window, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  project_close(appdata);
+  main_project_loaded(appdata);
+}
+
 static void 
 cb_menu_about(GtkWidget *window, gpointer data) {
   GtkAboutDialog *about = GTK_ABOUT_DIALOG(gtk_about_dialog_new());
@@ -279,9 +345,27 @@ void menu_create(appdata_t *appdata) {
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_zoomout), appdata);
 #endif
 
+  /* -------------------- Project submenu -------------------- */
+
+  item = gtk_menu_item_new_with_label( _("Project") );
+  gtk_menu_append(GTK_MENU_SHELL(menu), item);
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+
+  item = gtk_menu_item_new_with_label( _("Open...") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_project_open), 
+		   appdata);
+
+  appdata->menu_item_project_close = item = 
+    gtk_menu_item_new_with_label( _("Close") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_project_close), 
+		   appdata);
+
   /* -------------------- OSM submenu -------------------- */
 
-  item = gtk_menu_item_new_with_label( _("OSM") );
+  appdata->menu_osm = item = gtk_menu_item_new_with_label( _("OSM") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
@@ -303,7 +387,7 @@ void menu_create(appdata_t *appdata) {
 
   /* -------------------- wms submenu -------------------- */
 
-  item = gtk_menu_item_new_with_label( _("WMS") );
+  appdata->menu_wms = item = gtk_menu_item_new_with_label( _("WMS") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
@@ -329,7 +413,7 @@ void menu_create(appdata_t *appdata) {
 
   /* -------------------- map submenu -------------------- */
 
-  item = gtk_menu_item_new_with_label( _("Map") );
+  appdata->menu_map = item = gtk_menu_item_new_with_label( _("Map") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
@@ -350,19 +434,19 @@ void menu_create(appdata_t *appdata) {
 
   gtk_menu_append(GTK_MENU_SHELL(submenu), gtk_separator_menu_item_new());
 
-  item = gtk_menu_item_new_with_label( _("Redraw...") );
+  item = gtk_menu_item_new_with_label( _("Redraw") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_redraw), appdata);
 
   gtk_menu_append(GTK_MENU_SHELL(submenu), gtk_separator_menu_item_new());
 
-  item = gtk_menu_item_new_with_label( _("Style") );
+  item = gtk_menu_item_new_with_label( _("Style...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_style), appdata);
 
   /* -------------------- track submenu -------------------- */
 
-  item = gtk_menu_item_new_with_label( _("Track") );
+  appdata->track.menu_track = item = gtk_menu_item_new_with_label(_("Track"));
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
@@ -463,6 +547,12 @@ void cleanup(appdata_t *appdata) {
 
   settings_free(appdata->settings);
 
+  statusbar_free(appdata->statusbar);
+
+  iconbar_free(appdata->iconbar);
+
+  project_free(appdata->project);
+
   puts("everything is gone");
 }
 
@@ -508,7 +598,8 @@ gboolean on_window_key_press(GtkWidget *widget,
 #endif
   }
 
-  if(!handled) 
+  /* forward unprocessed key presses to map */
+  if(!handled && appdata->project && appdata->osm) 
     handled = map_key_press_event(appdata, event);
   
   return handled;
@@ -560,7 +651,8 @@ int main(int argc, char *argv[]) {
   gtk_window_set_title(GTK_WINDOW(appdata.window), "OSM2Go");
   /* Set a decent default size for the window. */
   gtk_window_set_default_size(GTK_WINDOW(appdata.window), 640, 480);
-  gtk_window_set_icon(GTK_WINDOW(appdata.window), icon_load(&appdata.icon, "osm2go"));
+  gtk_window_set_icon(GTK_WINDOW(appdata.window), 
+		      icon_load(&appdata.icon, PACKAGE));
 #endif
 
   g_signal_connect(G_OBJECT(appdata.window), "destroy", 
@@ -595,46 +687,11 @@ int main(int argc, char *argv[]) {
 
   appdata.presets = josm_presets_load();
 
-  /* -------------- load the project ------------- */
+  /* load project if one is specified in the settings */
+  if(appdata.settings->project)
+    project_load(&appdata, appdata.settings->project);
 
-  char *proj_name = project_select(&appdata);
-  if(!proj_name) {
-    cleanup(&appdata);
-    return 0;
-  }    
-
-  if(!project_open(&appdata, proj_name)) {
-    g_free(proj_name);
-    cleanup(&appdata);
-    return 0;
-  }    
-
-  g_free(proj_name);
-
-  if(!osm_sanity_check(GTK_WIDGET(appdata.window), appdata.osm)) {
-    cleanup(&appdata);
-    return 0;
-  }
-
-  printf("loading diff\n");
-  diff_restore(&appdata, appdata.project, appdata.osm);
-
-  map_init(&appdata);
-
-  appdata.track.track = track_restore(&appdata, appdata.project);
-  if(appdata.track.track)
-    map_track_draw(appdata.map, appdata.track.track);
-
-  wms_load(&appdata);
-
-  /* give main window a new name */
-#ifndef USE_HILDON
-  char *str = g_strdup_printf("OSM2Go - %s", appdata.project->name);
-  gtk_window_set_title(GTK_WINDOW(appdata.window), str);
-  g_free(str);
-#else
-  gtk_window_set_title(GTK_WINDOW(appdata.window), appdata.project->name);
-#endif
+  main_project_loaded(&appdata);
 
   /* ------------ jump into main loop ---------------- */
 
