@@ -21,6 +21,7 @@
 
 #include <gdk/gdkkeysyms.h>
 
+
 #undef DESTROY_WAIT_FOR_GTK
 
 static void map_statusbar(map_t *map, map_item_t *map_item) {
@@ -143,13 +144,18 @@ static void map_node_select(appdata_t *appdata, node_t *node) {
   memcpy(new_map_item, map_item, sizeof(map_item_t));
   new_map_item->highlight = TRUE;
 
-  float diameter = map->style->highlight.width + map->style->node.radius;
-  if(!node->ways) diameter += map->style->node.border_radius;
-  if(node->icon_buf && map->style->icon.enable) 
-    diameter = 16 * map->style->icon.scale;  // icon width = 24
+  float radius = map->style->highlight.width + map->style->node.radius;
+  if(!node->ways) radius += map->style->node.border_radius;
+  if(node->icon_buf && map->style->icon.enable) {
+    gint w = gdk_pixbuf_get_width(map_item->node->icon_buf);
+    gint h = gdk_pixbuf_get_height(map_item->node->icon_buf);
+    /* icons are technically square, so a radius slightly bigger */
+    /* than sqrt(2)*MAX(w,h) should fit nicely */
+    radius =  0.75 * map->style->icon.scale * ((w>h)?w:h);
+  }
 
-    map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
-		      x, y, diameter, map->style->highlight.color);
+  map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
+		    x, y, radius, map->style->highlight.color);
 
   if(!map_item->item) {
     /* and draw a fake node */
@@ -366,11 +372,13 @@ static canvas_item_t *map_node_new(map_t *map, node_t *node, gint radius,
   else
     map_item->item = canvas_image_new(map, CANVAS_GROUP_NODES, 
       node->icon_buf, 
-      node->lpos.x - map->style->icon.scale * 
-		      gdk_pixbuf_get_width(node->icon_buf)/2, 
-      node->lpos.y - map->style->icon.scale * 
-		      gdk_pixbuf_get_height(node->icon_buf)/2, 
+      node->lpos.x - map->style->icon.scale/2 * 
+		      gdk_pixbuf_get_width(node->icon_buf), 
+      node->lpos.y - map->style->icon.scale/2 * 
+		      gdk_pixbuf_get_height(node->icon_buf), 
 	      map->style->icon.scale,map->style->icon.scale);
+ 
+  canvas_item_set_zoom_max(map_item->item, node->zoom_max);
 
   /* attach map_item to nodes map_item_chain */
   map_item_chain_t **chain = &node->map_item_chain;
@@ -397,7 +405,9 @@ static canvas_item_t *map_way_single_new(map_t *map, way_t *way, gint radius,
   map_item->item = canvas_circle_new(map, CANVAS_GROUP_WAYS, 
 	  way->node_chain->node->lpos.x, way->node_chain->node->lpos.y, 
 				     radius, width, fill, border);
-  
+
+  // TODO: decide: do we need canvas_item_set_zoom_max() here too?
+
   /* attach map_item to nodes map_item_chain */
   map_item_chain_t **chain = &way->map_item_chain;
   while(*chain) chain = &(*chain)->next;
@@ -430,6 +440,8 @@ static canvas_item_t *map_way_new(map_t *map, canvas_group_t group,
     map_item->item = canvas_polyline_new(map, group, points, width, color);
   }
 
+  canvas_item_set_zoom_max(map_item->item, way->draw.zoom_max);
+
   /* attach map_item to ways map_item_chain */
   map_item_chain_t **chain = &way->map_item_chain;
   while(*chain) chain = &(*chain)->next;
@@ -540,8 +552,17 @@ static void map_item_remove(map_t *map, map_item_t *map_item) {
 }
 
 static void map_item_init(style_t *style, map_item_t *map_item) {
-  if(map_item->type == MAP_TYPE_WAY)
-    josm_elemstyles_colorize_way(style, map_item->way);
+  switch (map_item->type){
+    case MAP_TYPE_WAY:
+      josm_elemstyles_colorize_way(style, map_item->way);
+      break;
+    case MAP_TYPE_NODE:
+      josm_elemstyles_colorize_node(style, map_item->node);
+      break;
+    default:
+      g_assert((map_item->type == MAP_TYPE_NODE) ||
+	           (map_item->type == MAP_TYPE_WAY));
+  }
 }
 
 void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
@@ -652,9 +673,11 @@ void map_state_free(map_state_t *state) {
 }
 
 void map_free_map_item_chains(appdata_t *appdata) {
+  if(!appdata->osm) return;
+
 #ifndef DESTROY_WAIT_FOR_GTK
   printf("  DESTROY_WAIT_FOR_GTK not set, removing all chains now\n");
-  
+
   /* free all map_item_chains */
   node_t *node = appdata->osm->node;
   while(node) {
@@ -699,6 +722,18 @@ static gint map_destroy_event(GtkWidget *widget, gpointer data) {
   if(map->style)
     style_free(map->style);
 
+  /* destroy existing highlight */
+  if(map->highlight) {
+    printf("removing highlight\n");
+
+    map_highlight_t *hl = map->highlight;
+    while(hl) {
+      map_highlight_t *next = hl->next;
+      g_free(hl);
+      hl = next;
+    }
+  }
+
   g_free(map);
 
   appdata->map = NULL;
@@ -936,7 +971,26 @@ static void map_scroll_towards_centre(map_t *map, gdouble amt) {
 #endif // #if 0
 #endif // #ifdef USE_GOOCANVAS
 
-
+/* Deselects the current way or node if its zoom_max
+ * means that it's not going to render at the current map zoom. */
+void map_deselect_if_zoom_below_zoom_max(map_t *map) {
+    if (map->selected.type == MAP_TYPE_WAY) {
+        printf("will deselect way if zoomed below %f\n",
+               map->selected.way->draw.zoom_max);
+        if (map->state->zoom < map->selected.way->draw.zoom_max) {
+            printf("  deselecting way!\n");
+            map_item_deselect(map->appdata);
+        }
+    }
+    else if (map->selected.type == MAP_TYPE_NODE) {
+        printf("will deselect node if zoomed below %f\n",
+               map->selected.node->zoom_max);
+        if (map->state->zoom < map->selected.node->zoom_max) {
+            printf("  deselecting node!\n");
+            map_item_deselect(map->appdata);
+        }
+    }
+}
 
 void map_set_zoom(map_t *map, double zoom, 
 		  gboolean update_scroll_offsets) {
@@ -947,6 +1001,8 @@ void map_set_zoom(map_t *map, double zoom,
   map->state->zoom = zoom;
   canvas_set_zoom(map->canvas, map->state->zoom);
 
+  map_deselect_if_zoom_below_zoom_max(map);
+
   if (update_scroll_offsets) {
     if (!at_zoom_limit) {
       /* zooming affects the scroll offsets */
@@ -971,7 +1027,9 @@ static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 				 gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
 
-  if(event->type == GDK_SCROLL) {
+  if(!appdata->osm) return TRUE;
+
+  if(event->type == GDK_SCROLL && appdata->map && appdata->map->state) {
     if(event->direction) 
       map_set_zoom(appdata->map, 
 		   appdata->map->state->zoom / ZOOM_FACTOR_WHEEL, TRUE);
@@ -1561,7 +1619,7 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
       zoom = appdata->map->state->zoom;
       zoom *= ZOOM_FACTOR_BUTTON;
       map_set_zoom(appdata->map, zoom, TRUE);
-      printf("zoom is now %f\n", zoom);
+      printf("zoom is now %f (1:%d)\n", zoom, (int)zoom_to_scaledn(zoom));
       return TRUE;
       break;
 
@@ -1573,7 +1631,7 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
       zoom = appdata->map->state->zoom;
       zoom /= ZOOM_FACTOR_BUTTON;
       map_set_zoom(appdata->map, zoom, TRUE);
-      printf("zoom is now %f\n", zoom);
+      printf("zoom is now %f (1:%d)\n", zoom, (int)zoom_to_scaledn(zoom));
       return TRUE;
       break;
 
@@ -1642,7 +1700,7 @@ GtkWidget *map_new(appdata_t *appdata) {
 			| GDK_SCROLL_MASK
 			| GDK_POINTER_MOTION_MASK
     			| GDK_POINTER_MOTION_HINT_MASK);
-  
+
   gtk_signal_connect(GTK_OBJECT(map->canvas), 
      "button_press_event", G_CALLBACK(map_button_event), appdata);
   gtk_signal_connect(GTK_OBJECT(map->canvas), 
