@@ -151,6 +151,9 @@ static gboolean project_read(appdata_t *appdata,
     }
   }
   
+  xmlFreeDoc(doc);
+  xmlCleanupParser();
+
   /* ------------ set some default that may be missing ----------- */
   /* ------- e.g. from project files saved by old versions ------- */
   if(!project->wms_server)
@@ -237,6 +240,8 @@ gboolean project_save(GtkWidget *parent, project_t *project) {
 /* ------------ freeing projects --------------------- */
 
 void project_free(project_t *project) {
+  if(!project) return;
+
   if(project->name)       g_free(project->name);
   if(project->desc)       g_free(project->desc);
   if(project->server)     g_free(project->server);
@@ -1024,12 +1029,14 @@ gboolean project_open(appdata_t *appdata, char *name) {
 
   printf("project file = %s\n", project_file);
   if(!g_file_test(project_file, G_FILE_TEST_IS_REGULAR)) {
+    printf("requested project file doesn't exist\n");
     project_free(project);
     g_free(project_file);
     return FALSE;
   }
 
   if(!project_read(appdata, project_file, project)) {
+    printf("error reading project file\n");
     project_free(project);
     g_free(project_file);
     return FALSE;
@@ -1051,3 +1058,81 @@ gboolean project_open(appdata_t *appdata, char *name) {
 
   return TRUE;
 }
+
+gboolean project_close(appdata_t *appdata) {
+  if(!appdata->project) return FALSE;
+  
+  printf("closing current project\n");
+
+  /* redraw the entire map by destroying all map items and redrawing them */
+  if(appdata->osm)
+    diff_save(appdata->project, appdata->osm);
+
+  map_clear(appdata, MAP_LAYER_OBJECTS_ONLY);
+
+  if(appdata->osm) {
+    osm_free(&appdata->icon, appdata->osm);
+    appdata->osm = NULL;
+  }
+
+  project_free(appdata->project);
+  appdata->project = NULL;
+
+  return TRUE;
+}
+
+gboolean project_load(appdata_t *appdata, char *name) {
+  char *proj_name = NULL;
+
+  if(!name) {
+    /* make user select a project */
+    proj_name = project_select(appdata);
+    if(!proj_name) {
+      printf("no project selected\n");
+      return FALSE;
+    }
+  } else
+    proj_name = g_strdup(name);
+
+  /* close current project */
+  if(appdata->project) 
+    project_close(appdata);
+
+  /* open project itself */
+  if(!project_open(appdata, proj_name)) {
+    printf("error opening requested project\n");
+    g_free(proj_name);
+    return FALSE;
+  }    
+
+  g_free(proj_name);
+
+  /* check if OSM data is valid */
+  if(!osm_sanity_check(GTK_WIDGET(appdata->window), appdata->osm)) {
+    printf("project/osm sanity checks failed, unloading project\n");
+    project_free(appdata->project);
+    return FALSE;
+  }
+
+  /* load diff possibly preset */
+  diff_restore(appdata, appdata->project, appdata->osm);
+  
+  /* prepare colors etc, draw data and adjust scroll/zoom settings */
+  map_init(appdata);
+
+  /* restore a track */
+  appdata->track.track = track_restore(appdata, appdata->project);
+  if(appdata->track.track)
+    map_track_draw(appdata->map, appdata->track.track);
+
+  /* finally load a background if present */
+  wms_load(appdata);
+
+  /* save the name of the project for the perferences */
+  if(appdata->settings->project)
+    g_free(appdata->settings->project);
+  appdata->settings->project = g_strdup(appdata->project->name);
+
+  return TRUE;
+}
+
